SHAPES = [
    "round",
    "square",
    "triangular",
    "circular",
    "oval",
    "rectangular",
    "spherical",
    "oblong",
    "flat",
    "curved",
    "straight",
    "twisted",
    "hexagonal",
    "octagonal",
    # "diamond-shaped",
    "conical",
    "pyramidal",
    "cylindrical",
    "spiral",
    "elliptical",
    "linear",
    "angular",
    # "star-shaped",
    "zigzag",
    # "heart-shaped",
    # "wave-like",
    "bent",
    "irregular",
    "triangulate",
    "cubical",
    "serpentine",
    "blocky",
    "orbicular",
    "arched",
    "pointed",
    "knotted",
    "fractal",
    "tapered",
    "shapeless",
    # "egg-shaped",
    "crescent",
    "narrowed",
    "hemispherical",
    "geometric",
    "squared",
    "meandering",
    # "sharp-edged",
    "wavy",
]



"""These tests validate the words that are used for hrid generation."""

from collections import Counter

import pytest

from humid.core import ORDER


def test_no_duplicates():
    """Test whether there are any duplicates per category."""
    for list_of_words in ORDER:
        duplicates = [
            item for item, count in Counter(list_of_words).items() if count > 1
        ]
        print(f"{duplicates=}")
        assert len(list_of_words) == len(set(list_of_words))


def test_no_duplicates_between():
    """Test whether there are any duplicates in total."""
    all_words = [item for sublist in ORDER for item in sublist]
    duplicates = [item for item, count in Counter(all_words).items() if count > 1]
    print(f"{duplicates=}")
    assert len(all_words) == len(set(all_words))


def test_no_spaces_in_words():
    """Test whether there are undesired spaces in any of the words."""
    all_words = [item for sublist in ORDER for item in sublist]
    words_with_spaces = []

    for word in all_words:
        if " " in word:
            words_with_spaces.append(word)

    print(f"{words_with_spaces=}")
    assert len(words_with_spaces) == 0


def test_no_dashses_in_words():
    """Test whether there are undesired dashes in any of the words."""
    all_words = [item for sublist in ORDER for item in sublist]
    words_with_dashes = []

    for word in all_words:
        if "-" in word:
            words_with_dashes.append(word)

    print(f"{words_with_dashes=}")
    assert len(words_with_dashes) == 0


WHITELIST = {"amberjack", "newt", "goldenretriever", "greatdane", "badger"}


def test_later_word_does_not_start_with_earlier_word():
    """Checks if a word is not an existing word with a suffix.But only checks this for words that can follow eachother in the hrid based on their order.

    Example:
        avoid `black-blackbird`
    """
    starting_with_other_word: list[tuple[str, str]] = []

    for i in range(len(ORDER) + 1):
        if i == len(ORDER) - 1:
            break

        first = ORDER[i]
        second = ORDER[i + 1]

        for first_word in first:
            for second_word in second:
                if second_word.startswith(first_word):
                    if second_word in WHITELIST:
                        continue
                    starting_with_other_word.append((first_word, second_word))

    print(starting_with_other_word)
    assert len(starting_with_other_word) == 0


@pytest.mark.skip("Actually this is ok.")
def test_earlier_word_does_not_end_in_later_word():
    """Checks if a word is not an existing word with a suffix.But only checks this for words that can follow eachother in the hrid based on their order.

    Example:
        avoid `elegant-ant`
    """
    ending_with_later_word: list[tuple[str, str]] = []

    for i in range(len(ORDER) + 1):
        if i == len(ORDER) - 1:
            break

        first = ORDER[i]
        second = ORDER[i + 1]

        for first_word in first:
            for second_word in second:
                if first_word.endswith(second_word):
                    ending_with_later_word.append((first_word, second_word))

    print(ending_with_later_word)
    assert len(ending_with_later_word) == 0


@pytest.mark.skip("TODO")
def test_minimum_human_readable_part_length():
    """Check whether the minimum length of the human readable part is within bounds."""
    raise AssertionError()


@pytest.mark.skip("TODO")
def test_maximum_human_readable_part_length():
    """Check whether the maximum length of the human readable part is within bounds."""
    raise AssertionError()



"""The core of the humid library."""

import random
import string

from ._words import W

ORDER = [
    # first part
    W["opinions"] + W["sizes"] + W["ages"] + W["shapes"] + W["colors"] + W["origins"],
    # second part
    W["pokemons"] + W["animals"],
]
"""The order in which the hrid is constructed."""

SEPARATOR = "-"
"""The separator for the hrid"""

CHARACTERS = string.ascii_letters + string.digits
"""The characters used for generating the identifier"""


def _make_identifier(length: int = 22) -> str:
    """Generate a random alphanumeric string.

    Returns:
        str: The random string.
    """
    return "".join(random.choices(CHARACTERS, k=length))


def hrid() -> str:
    """Returns a default human readable identifier.

    Returns:
        str: The human readable identifier based on elements in `ORDER`.

    Examples:
        >>> hrid()
        'modest-tarpon-oSYCzJPazgTg94KqiEb392'
    """
    selection: list[str] = []
    for words in ORDER:
        selection.append(random.choice(words))

    """Add the identifier string"""
    selection.append(_make_identifier())

    the_id = SEPARATOR.join(selection)

    return the_id



"""The command line interface for humid."""

import argparse

from ._words import W
from .core import hrid


def generate():
    """Runs the cli for generating new hrid's."""
    parser = argparse.ArgumentParser(description="humid generates human readable ids.")
    parser.add_argument(
        "-n", "--number", type=int, help="The number of hrids to generate.", default=1
    )

    args = parser.parse_args()

    for _ in range(args.number):
        print(hrid())


def check_word():
    """Checks new words against existing library."""
    parser = argparse.ArgumentParser(
        description="Check if a word already in the humid dictionary."
    )
    parser.add_argument(
        "-w", "--word", type=str, help="The word to check against the existing library."
    )

    args = parser.parse_args()

    all_words = [word for sublist in W.values() for word in sublist]

    if args.word:
        print(f"Word already in humid dictionary: {args.word in all_words}")


def print_as_list(strings: list[str]):
    """Print a list of words as string so it can be copy pasted into one of the files in `src/humid/words/`."""
    for element in strings:
        print(f'"{element}",')



POKEMONS = [
    "Bulbasaur",
    "Ivysaur",
    "Venusaur",
    "Charmander",
    "Charmeleon",
    "Charizard",
    "Squirtle",
    "Wartortle",
    "Blastoise",
    "Caterpie",
    "Metapod",
    "Butterfree",
    "Weedle",
    "Kakuna",
    "Beedrill",
    "Pidgey",
    "Pidgeotto",
    "Pidgeot",
    "Rattata",
    "Raticate",
    "Spearow",
    "Fearow",
    "Ekans",
    "Arbok",
    "Pikachu",
    "Raichu",
    "Sandshrew",
    "Sandslash",
    # "Nidoran♀",
    "Nidoran",
    "Nidorina",
    "Nidoqueen",
    # "Nidoran♂",
    "Nidorino",
    "Nidoking",
    "Clefairy",
    "Clefable",
    "Vulpix",
    "Ninetales",
    "Jigglypuff",
    "Wigglytuff",
    "Zubat",
    "Golbat",
    "Oddish",
    "Gloom",
    "Vileplume",
    "Paras",
    "Parasect",
    "Venonat",
    "Venomoth",
    "Diglett",
    "Dugtrio",
    "Meowth",
    "Persian",
    "Psyduck",
    "Golduck",
    "Mankey",
    "Primeape",
    "Growlithe",
    "Arcanine",
    "Poliwag",
    "Poliwhirl",
    "Poliwrath",
    "Abra",
    "Kadabra",
    "Alakazam",
    "Machop",
    "Machoke",
    "Machamp",
    "Bellsprout",
    "Weepinbell",
    "Victreebel",
    "Tentacool",
    "Tentacruel",
    "Geodude",
    "Graveler",
    "Golem",
    "Ponyta",
    "Rapidash",
    "Slowpoke",
    "Slowbro",
    "Magnemite",
    "Magneton",
    "Farfetch'd",
    "Doduo",
    "Dodrio",
    "Seel",
    "Dewgong",
    "Grimer",
    "Muk",
    "Shellder",
    "Cloyster",
    "Gastly",
    "Haunter",
    "Gengar",
    "Onix",
    "Drowzee",
    "Hypno",
    "Krabby",
    "Kingler",
    "Voltorb",
    "Electrode",
    "Exeggcute",
    "Exeggutor",
    "Cubone",
    "Marowak",
    "Hitmonlee",
    "Hitmonchan",
    "Lickitung",
    "Koffing",
    "Weezing",
    "Rhyhorn",
    "Rhydon",
    "Chansey",
    "Tangela",
    "Kangaskhan",
    "Horsea",
    "Seadra",
    "Goldeen",
    "Seaking",
    "Staryu",
    "Starmie",
    "Mr.Mime",
    "Scyther",
    "Jynx",
    "Electabuzz",
    "Magmar",
    "Pinsir",
    "Tauros",
    "Magikarp",
    "Gyarados",
    "Lapras",
    "Ditto",
    "Eevee",
    "Vaporeon",
    "Jolteon",
    "Flareon",
    "Porygon",
    "Omanyte",
    "Omastar",
    "Kabuto",
    "Kabutops",
    "Aerodactyl",
    "Snorlax",
    "Articuno",
    "Zapdos",
    "Moltres",
    "Dratini",
    "Dragonair",
    "Dragonite",
    "Mewtwo",
    "Mew",
    "Chikorita",
    "Bayleef",
    "Meganium",
    "Cyndaquil",
    "Quilava",
    "Typhlosion",
    "Totodile",
    "Croconaw",
    "Feraligatr",
    "Sentret",
    "Furret",
    "Hoothoot",
    "Noctowl",
    "Ledyba",
    "Ledian",
    "Spinarak",
    "Ariados",
    "Crobat",
    "Chinchou",
    "Lanturn",
    "Pichu",
    "Cleffa",
    "Igglybuff",
    "Togepi",
    "Togetic",
    "Natu",
    "Xatu",
    "Mareep",
    "Flaaffy",
    "Ampharos",
    "Bellossom",
    "Marill",
    "Azumarill",
    "Sudowoodo",
    "Politoed",
    "Hoppip",
    "Skiploom",
    "Jumpluff",
    "Aipom",
    "Sunkern",
    "Sunflora",
    "Yanma",
    "Wooper",
    "Quagsire",
    "Espeon",
    "Umbreon",
    "Murkrow",
    "Slowking",
    "Misdreavus",
    "Unown",
    "Wobbuffet",
    "Girafarig",
    "Pineco",
    "Forretress",
    "Dunsparce",
    "Gligar",
    "Steelix",
    "Snubbull",
    "Granbull",
    "Qwilfish",
    "Scizor",
    "Shuckle",
    "Heracross",
    "Sneasel",
    "Teddiursa",
    "Ursaring",
    "Slugma",
    "Magcargo",
    "Swinub",
    "Piloswine",
    "Corsola",
    "Remoraid",
    "Octillery",
    "Delibird",
    "Mantine",
    "Skarmory",
    "Houndour",
    "Houndoom",
    "Kingdra",
    "Phanpy",
    "Donphan",
    "Porygon2",
    "Stantler",
    "Smeargle",
    "Tyrogue",
    "Hitmontop",
    "Smoochum",
    "Elekid",
    "Magby",
    "Miltank",
    "Blissey",
    "Raikou",
    "Entei",
    "Suicune",
    "Larvitar",
    "Pupitar",
    "Tyranitar",
    "Lugia",
    # "Ho-oh",
    "Celebi",
    "Treecko",
    "Grovyle",
    "Sceptile",
    "Torchic",
    "Combusken",
    "Blaziken",
    "Mudkip",
    "Marshtomp",
    "Swampert",
    "Poochyena",
    "Mightyena",
    "Zigzagoon",
    "Linoone",
    "Wurmple",
    "Silcoon",
    "Beautifly",
    "Cascoon",
    "Dustox",
    "Lotad",
    "Lombre",
    "Ludicolo",
    "Seedot",
    "Nuzleaf",
    "Shiftry",
    "Taillow",
    "Swellow",
    "Wingull",
    "Pelipper",
    "Ralts",
    "Kirlia",
    "Gardevoir",
    "Surskit",
    "Masquerain",
    "Shroomish",
    "Breloom",
    "Slakoth",
    "Vigoroth",
    "Slaking",
    "Nincada",
    "Ninjask",
    "Shedinja",
    "Whismur",
    "Loudred",
    "Exploud",
    "Makuhita",
    "Hariyama",
    "Azurill",
    "Nosepass",
    "Skitty",
    "Delcatty",
    "Sableye",
    "Mawile",
    "Aron",
    "Lairon",
    "Aggron",
    "Meditite",
    "Medicham",
    "Electrike",
    "Manectric",
    "Plusle",
    "Minun",
    "Volbeat",
    "Illumise",
    "Roselia",
    "Gulpin",
    "Swalot",
    "Carvanha",
    "Sharpedo",
    "Wailmer",
    "Wailord",
    "Numel",
    "Camerupt",
    "Torkoal",
    "Spoink",
    "Grumpig",
    "Spinda",
    "Trapinch",
    "Vibrava",
    "Flygon",
    "Cacnea",
    "Cacturne",
    "Swablu",
    "Altaria",
    "Zangoose",
    "Seviper",
    "Lunatone",
    "Solrock",
    "Barboach",
    "Whiscash",
    "Corphish",
    "Crawdaunt",
    "Baltoy",
    "Claydol",
    "Lileep",
    "Cradily",
    "Anorith",
    "Armaldo",
    "Feebas",
    "Milotic",
    "Castform",
    "Kecleon",
    "Shuppet",
    "Banette",
    "Duskull",
    "Dusclops",
    "Tropius",
    "Chimecho",
    "Absol",
    "Wynaut",
    "Snorunt",
    "Glalie",
    "Spheal",
    "Sealeo",
    "Walrein",
    "Clamperl",
    "Huntail",
    "Gorebyss",
    "Relicanth",
    "Luvdisc",
    "Bagon",
    "Shelgon",
    "Salamence",
    "Beldum",
    "Metang",
    "Metagross",
    "Regirock",
    "Regice",
    "Registeel",
    "Latias",
    "Latios",
    "Kyogre",
    "Groudon",
    "Rayquaza",
    "Jirachi",
    "Deoxys",
    "Turtwig",
    "Grotle",
    "Torterra",
    "Chimchar",
    "Monferno",
    "Infernape",
    "Piplup",
    "Prinplup",
    "Empoleon",
    "Starly",
    "Staravia",
    "Staraptor",
    "Bidoof",
    "Bibarel",
    "Kricketot",
    "Kricketune",
    "Shinx",
    "Luxio",
    "Luxray",
    "Budew",
    "Roserade",
    "Cranidos",
    "Rampardos",
    "Shieldon",
    "Bastiodon",
    "Burmy",
    "Wormadam",
    "Mothim",
    "Combee",
    "Vespiquen",
    "Pachirisu",
    "Buizel",
    "Floatzel",
    "Cherubi",
    "Cherrim",
]



MATERIALS = [
    "wooden",
    "plastic",
    "metallic",
    "silk",
    "cotton",
    "leather",
    "glass",
    "ceramic",
    "steel",
    "iron",
    "bronze",
    "gold",
    "silver",
    "aluminum",
    "stone",
    "marble",
    "copper",
    "woolen",
    "rubber",
    "nylon",
    "concrete",
    "brick",
    "paper",
    "bamboo",
    "tin",
    "pewter",
    "ivory",
    "porcelain",
    "fiber",
    "acrylic",
    "velvet",
    "linen",
    "suede",
    "cardboard",
    "graphite",
    "platinum",
    "granite",
    "crystal",
    "canvas",
    "titanium",
    "resin",
    "charcoal",
    "oak",
    "maple",
    "ebony",
    "pine",
    "birch",
    "carbon",
    "synthetic",
    "felt",
]



AGES = [
    "old",
    "young",
    "new",
    "ancient",
    "modern",
    "elderly",
    # "middle-aged",
    "mature",
    "youthful",
    "teenage",
    "baby",
    "infant",
    "prehistoric",
    "medieval",
    "contemporary",
    "antique",
    "vintage",
    "recent",
    "fresh",
    "early",
    "late",
    "dated",
    "juvenile",
    "adolescent",
    "ripe",
    "senior",
    "current",
    "neonatal",
    "futuristic",
    "premature",
    "past",
    "historic",
    "geriatric",
    "archaic",
    "primeval",
    "immortal",
    "timeless",
    "regal",
    "prodigious",
    "rusty",
    "sprightly",
    "neoteric",
    # "age-old",
    "midlife",
    "childhood",
    # "ancient-time",
    "millennial",
]



from .age import AGES
from .animals import ANIMALS
from .colors import COLORS
from .opininions import OPINIONS
from .origin import ORIGINS
from .pokemon import POKEMONS
from .shape import SHAPES
from .size import SIZES

W: dict[str, list[str]] = {
    # adjectives
    "ages": AGES,
    "colors": COLORS,
    "opinions": OPINIONS,
    "origins": ORIGINS,
    "shapes": SHAPES,
    "sizes": SIZES,
    # Nouns
    "animals": ANIMALS,
    "pokemons": POKEMONS,
}
"""All the words that are available for generation of hrid's"""

__all__ = ["W"]



"""humid helps generating human friendly identifiers."""

from .core import hrid

__all__ = ["hrid"]

# Note that adjectives should always go in this order in the english language.
# adjective_order = [
#     QUANTITIES,
#     OPINIONS,
#     SIZES,
#     AGES,
#     SHAPES,
#     COLORS,
#     ORIGINS,
#     MATERIALS,
#     PURPOSES,
#     NAMES,
# ]



PURPOSES = [
    "sleeping",
    "cooking",
    "writing",
    "running",
    "driving",
    "gardening",
    "swimming",
    "reading",
    "dining",
    "camping",
    "cleaning",
    "baking",
    "dancing",
    "painting",
    "training",
    "fishing",
    "hiking",
    "decorating",
    "shooting",
    "hunting",
    "gaming",
    "climbing",
    "wrestling",
    "surfing",
    "skating",
    "skiing",
    "traveling",
    "shopping",
    "racing",
    "flying",
    "cycling",
    "photography",
    "training",
    "sewing",
    "jogging",
    "boating",
    "kayaking",
    "snowboarding",
    "parachuting",
    "knitting",
    "pottery",
    "yoga",
    "exercise",
    "crafting",
    "playwriting",
    "screenwriting",
    "bodybuilding",
    "pilates",
    "woodworking",
    "ceramics",
]



COLORS = [
    "red",
    "blue",
    "green",
    "yellow",
    "purple",
    "orange",
    "pink",
    "brown",
    "black",
    "white",
    "gray",
    "golden",
    "silver",
    "bronze",
    "turquoise",
    "amber",
    "crimson",
    "violet",
    "magenta",
    "indigo",
    "maroon",
    "scarlet",
    "teal",
    "beige",
    "navy",
    "lavender",
    "peach",
    "fuchsia",
    "emerald",
    "aqua",
    "ruby",
    "sapphire",
    "chartreuse",
    "plum",
    "lime",
    "cobalt",
    "jade",
    "cinnamon",
    "rose",
    "tan",
    "mint",
    "cherry",
    "mauve",
    "khaki",
    "burgundy",
    "copper",
    "apricot",
    "cerulean",
]



"""These tests validate probability of generating colliding hrids."""

from humid import hrid


def test_no_duplicate_id():
    """Test whether no collisions are encountered when generating N times."""
    TIMES_TO_GENERATE = 1_000_000
    ids = set()

    for i in range(TIMES_TO_GENERATE):
        the_id = hrid()

        if the_id in ids:
            raise ValueError(f"iter {i}: {the_id} already present in set")

        ids.add(the_id)



ORIGINS = [
    "American",
    "French",
    "Italian",
    "Chinese",
    "Indian",
    "Mexican",
    "British",
    "Japanese",
    "Russian",
    "Spanish",
    "Brazilian",
    "Egyptian",
    "German",
    "Australian",
    "Greek",
    "Turkish",
    # "Persian",
    "Nigerian",
    "Scottish",
    "Irish",
    "Swedish",
    "Swiss",
    "Thai",
    "Korean",
    "Vietnamese",
    "Cuban",
    "Colombian",
    "Argentine",
    "Moroccan",
    "Portuguese",
    "Dutch",
    "Norwegian",
    "Danish",
    "Filipino",
    "Indonesian",
    "Malaysian",
    "Pakistani",
    "Afghan",
    "Arab",
    "Jamaican",
    "Polish",
    "Romanian",
    "Finnish",
    "Hungarian",
    "Ukrainian",
    # "Sri Lankan",
    "Mongolian",
    "Welsh",
    "Lithuanian",
    "Chilean",
]



ANIMALS = [
    "aardvark",
    "aardwolf",
    "abac",
    "abacusfish",
    "abalone",
    # "acornwoodpecker",
    "affenpinscher",
    # "affricanpotto",
    # "africanbullfrog",
    # "africanclawedfrog",
    # "africanelephant",
    # "africangrey",
    "agama",
    "agouti",
    "airedale",
    # "airedaleterrier",
    "akbac",
    "akbash",
    "akita",
    "albatross",
    "albino",
    "alligator",
    # "alligatorgar",
    "alpaca",
    "alpakka",
    "alpine",
    "amberjack",
    "anaconda",
    "angelfish",
    "ant",
    "antbear",
    "antbird",
    "anteater",
    "antelope",
    "aoudad",
    "apalm",
    "ape",
    "arapaima",
    # "arcticfox",
    "armadillo",
    # "asianhornet",
    # "atlanticmackerel",
    # "australiancattledog",
    # "australianshepherd",
    # "australianterrier",
    "avocet",
    "axolotl",
    "babirusa",
    "baboon",
    "bacchus",
    "bactrian",
    "badger",
    "bagworm",
    # "bald eagle",
    "baldcypress",
    "baldeagle",
    "bandicoot",
    "barb",
    "barbet",
    "barnacle",
    "barnowl",
    "barracuda",
    "barramundi",
    "barredowl",
    "basset",
    "bat",
    "bear",
    "bearcat",
    "beardeddragon",
    "beardedseal",
    "beardedvulture",
    "beaver",
    "beetle",
    "beetlebug",
    "beluga",
    "bernhardiner",
    "bilby",
    "binturong",
    "birch",
    "bird",
    "bison",
    # "black bear",
    # "blackbear",
    # "blackbird",
    "bloodhound",
    "blowfish",
    # "bluejay",
    "boa",
    # "boa constrictor",
    "boar",
    "bobcat",
    "bobolink",
    "bongo",
    "bonito",
    "booby",
    # "bostonterrier",
    "brambling",
    # "brownbear",
    "buffalo",
    "bulldog",
    "bullfrog",
    "bullmoose",
    "burro",
    "butterfly",
    "butterflyfish",
    "caballero",
    "caballo",
    "caballus",
    # "cactuswren",
    "caiman",
    "cairn",
    "cairnterrier",
    "calf",
    "camel",
    "canary",
    "capuchin",
    "capybara",
    "caracal",
    "cardinal",
    "caribou",
    "carp",
    "cassowary",
    "cat",
    "catbird",
    "caterpillar",
    "catfish",
    "cattle",
    "cavalier",
    "centaur",
    "centipede",
    "chameleon",
    "cheetah",
    "chickadee",
    "chimera",
    "chimpanzee",
    "chinchilla",
    "chipmunk",
    "cichlid",
    "civet",
    "clams",
    "clownfish",
    "coati",
    "cobra",
    "cobratop",
    "cockatoo",
    "cockroach",
    # "collaredlizard",
    "collie",
    "commonseal",
    "condor",
    "coot",
    "coral",
    "cougar",
    "cow",
    "cowbird",
    "coyote",
    "coypu",
    "crab",
    "crane",
    "crestedcrane",
    # "crimsonwing",
    "crocodile",
    "crocodilefish",
    "crow",
    "cuckoo",
    "cuttlefish",
    "dachshund",
    "dachsund",
    "daffodil",
    "dalmatian",
    "dalmation",
    "dama",
    "deer",
    "deerhound",
    "dingo",
    "dodo",
    "dog",
    "dogfish",
    "dolphin",
    "dolphinfish",
    # "domesticanimal",
    "donkey",
    "dove",
    "dragonfly",
    "dromedary",
    "duck",
    "duckling",
    "dugong",
    "dungbeetle",
    "eagle",
    # "eagleowl",
    "earthworm",
    "earwig",
    "echidna",
    "eel",
    "elephant",
    "elephantseal",
    "elk",
    "emu",
    "entlebucher",
    "erlang",
    "euro",
    "eurohound",
    "falcon",
    "falkland",
    "fawn",
    "fennec",
    # "fennecfox",
    "ferret",
    "finch",
    "fish",
    "fishingcat",
    "flamingo",
    "flicker",
    "fly",
    "flyingfox",
    "fossa",
    "fox",
    "foxhound",
    "foxterrier",
    "frenchbulldog",
    "frilledlizard",
    "frilledshark",
    "frog",
    "frogfish",
    "fugu",
    "galago",
    "gander",
    "gazelle",
    "gecko",
    "gerbil",
    # "giantpanda",
    "gibbon",
    "gila",
    "giraffe",
    "glenofimal",
    "gnu",
    "goat",
    "goatfish",
    # "goldenretriever",
    "goldfish",
    "goose",
    "goosefish",
    "gopher",
    "goral",
    "gorilla",
    # "gorillagrouper",
    "grasshopper",
    "greatdane",
    # "greatwhite",
    "grebe",
    "grizzlybear",
    # "grizzly bear",
    "groundhog",
    "guanaco",
    # "guinea",
    "guineapig",
    "gull",
    "guppy",
    "hamadryas",
    "hamster",
    # "harbor seal",
    "hare",
    "harlequin",
    "harpy",
    "hawk",
    "hedgehog",
    "hen",
    "hermitcrab",
    "heron",
    "herring",
    "hippopotamus",
    # "honey badger",
    "honeybadger",
    "honeybee",
    "hornedtoad",
    "hornet",
    "horse",
    "horseshoecrab",
    "hound",
    "human",
    "hummingbird",
    "hyacinth",
    "hyena",
    "hyrax",
    "ibex",
    "ibis",
    "iguana",
    "impala",
    "inchworm",
    "indri",
    "insect",
    "irishsetter",
    "jackal",
    "jackrabbit",
    "jaguar",
    "jaguarundi",
    "jalapeno",
    "jellyfish",
    "jerboa",
    "junglecat",
    "junglefowl",
    "kakapo",
    "kangaroo",
    "kennel",
    "kestrel",
    "killerwhale",
    "kingfisher",
    "kinkajou",
    "kiwi",
    "koala",
    "kodiakbear",
    "komodo",
    # "komodo dragon",
    "komododragon",
    "kookaburra",
    "kraken",
    "krill",
    "kudu",
    "labrador",
    "ladybug",
    "lamb",
    "langur",
    "lemming",
    "lemonshark",
    "lemur",
    "leopard",
    "leopardseal",
    "liger",
    "linnet",
    "lion",
    "lizard",
    "llama",
    "lobster",
    # "longhorn",
    "lori",
    "lorikeet",
    "loris",
    "louse",
    "luna",
    "lynx",
    "lyrebird",
    "maggot",
    "magpie",
    "malayanbear",
    "mallard",
    "manatee",
    "mandrill",
    "manedwolf",
    "mantid",
    "mantis",
    "marabou",
    "marmot",
    "marsupial",
    "martin",
    "meerkat",
    "merlin",
    "millipede",
    "mink",
    "minke",
    "mockingbird",
    "mole",
    "monarch",
    "mongoose",
    "monkey",
    "monkfish",
    "moose",
    "moth",
    "mountainlion",
    "mouse",
    "mouselemur",
    "muskox",
    "muskrat",
    "napoleonfish",
    "narwhal",
    "neonfish",
    "newt",
    "nightjar",
    "numbat",
    "ocelot",
    "octopus",
    "okapi",
    "opossum",
    "orangutan",
    "orca",
    "ostrich",
    "otter",
    "owl",
    "ox",
    # "oxpecker",
    "panda",
    "pangolin",
    "parakeet",
    "parrot",
    "parrotfish",
    # "pato",
    "peacock",
    "pelican",
    "penguin",
    "pequena",
    "perch",
    "pigeon",
    "platypus",
    "polarbear",
    "porcupine",
    "possum",
    "pufferfish",
    "puffin",
    "puma",
    "python",
    "quail",
    "quokka",
    "quoll",
    "rabbit",
    "raccoon",
    "rat",
    "rattlesnake",
    "raven",
    # "redfox",
    # "redpanda",
    "reindeer",
    "renard",
    "rhino",
    "rhinoceros",
    "ringtail",
    "robin",
    "rockhopper",
    "rooster",
    "salamander",
    "salmon",
    "sandpiper",
    "sandshark",
    "sawfish",
    "scalyanteater",
    "scorpion",
    # "sea lion",
    "seagull",
    "seahorse",
    "seal",
    "sealion",
    "seaotter",
    "seaurchin",
    "serval",
    "shark",
    "shearwater",
    "sheep",
    "shelduck",
    "shrike",
    "shrimp",
    "sifaka",
    "siskin",
    "skimmer",
    "skink",
    "skua",
    "skunk",
    "sloth",
    "slothbear",
    "snail",
    "snake",
    "snappingturtle",
    "snowleopard",
    "snowyowl",
    "sombrero",
    "sparrow",
    "spatula",
    "spider",
    "spinyanteater",
    "squid",
    "squill",
    "squirrel",
    "squirrelfish",
    "starfish",
    "starling",
    "stingray",
    "stoat",
    "stork",
    "sturgeon",
    "sunbear",
    "swallow",
    "swan",
    "tamarin",
    "tamarind",
    "tapir",
    "tarantula",
    "tarpon",
    "tarsier",
    "termite",
    "terrapin",
    "terrier",
    "thrush",
    "tiger",
    "toad",
    "toadfish",
    "tortoise",
    "toucan",
    "treecreeper",
    "treefrog",
    "trout",
    "tuatara",
    "tuna",
    "turkey",
    "turtle",
    "unicornfish",
    "urchin",
    "viper",
    "vole",
    "vulture",
    "wagtail",
    "wallaby",
    "walrus",
    "warthog",
    "wasp",
    "waterbuck",
    "waterbuffalo",
    "weasel",
    "whale",
    "wildebeest",
    "wolf",
    "wolverine",
    "wombat",
    "woodchuck",
    "woodcock",
    "woodpecker",
    "worm",
    "wren",
    "yak",
    "zander",
    "zebra",
    "zebrafish",
    "zebu",
    "zorilla",
    "zorse",
    "zuchon",
    "zuniga",
]



"""Check if the cli works as intended."""

import subprocess


def test_humid_cli():
    """Can we run the help flag."""
    # Call the CLI tool using subprocess
    result = subprocess.run(
        ["hrid", "--help"], capture_output=True, text=True, check=True
    )

    # Check the exit code
    assert result.returncode == 0, "CLI tool did not exit with code 0"

    # Check that the output contains the expected help message or some part of it
    assert "usage" in result.stdout, "CLI tool did not output the expected help message"



"""These tests validate the basic workings of the package."""

from humid import hrid


def test_basic():
    """Test whether a hrid is outputted of the correct type."""
    my_id = hrid()
    assert isinstance(my_id, str)



"""Compare the performance of humid vs std lib UUIDs."""

import statistics
import time
from collections.abc import Callable
from typing import TypeVar
from uuid import UUID, uuid4

from uniplot import histogram

from humid import hrid

T = TypeVar("T", UUID, str)  # Define a type variable that can be either UUID or str


def _compare(uuids: list[T]) -> bool:
    for i in range(len(uuids)):
        for j in range(i + 1, len(uuids)):
            if uuids[i] == uuids[j]:
                print("Oh no collision")
                return False

    return True


def _obtain_durations(
    uuid_constructor: Callable[[], T],
    nr_uuids: int,
    n_experiments: int,
    verbose=False,
) -> float:
    uuids = [uuid_constructor() for _ in range(nr_uuids)]
    durations = []

    # Perform pairwise comparison
    for i in range(n_experiments):
        if verbose:
            print(f"Starting experiment {i}", end="\r")

        start = time.perf_counter()
        _compare(uuids)
        end = time.perf_counter()

        duration = end - start
        durations.append(duration)

    avg_duration = statistics.mean(durations)

    if verbose:
        print()
        print(f"Average duration: {avg_duration}")

        histogram(
            durations,
            bins_min=min(durations),
            bins_max=max(durations),
            title=f"Duration Histogram {n_experiments} samples and {nr_uuids} UUIDS",
            x_unit="Sec",
            y_unit="#",
        )
    return avg_duration


def test_hrid_comparison_faster_than_uuid4_comparison():
    """Test comparison of hrid's is at least same speed or greater than uuid4's."""
    N_EXPERIMENTS = 100

    uuid4_avg_duration = _obtain_durations(
        uuid_constructor=uuid4, nr_uuids=1000, n_experiments=N_EXPERIMENTS
    )
    hrid_avg_duration = _obtain_durations(
        uuid_constructor=hrid, nr_uuids=1000, n_experiments=N_EXPERIMENTS
    )

    print(f"uuid4 avg duration: {uuid4_avg_duration}")
    print(f"hrid avg duration: {hrid_avg_duration}")

    assert hrid_avg_duration <= uuid4_avg_duration



QUANTITIES = [
    "one",
    "two",
    "three",
    "several",
    "many",
    "few",
    "little",
    "numerous",
    "couple",
    "some",
    "various",
    "dozen",
    "hundred",
    "thousand",
    "million",
    "countless",
    "myriad",
    "multiple",
    "single",
    "fewest",
    "plenty",
    "half",
    "fewer",
    "all",
    "none",
    "zero",
    "innumerable",
    "a lot",
    "a bit",
    "ample",
    "pair",
    "double",
    "quadruple",
    "fifty",
    "sixteen",
    "ninety",
    "sixty",
    "hundreds",
    "dozens",
    "a dozen",
    "a handful",
    "loads",
    "oodles",
    "tons",
    "thousands",
    "trillion",
    "billions",
    "a pair",
    "double",
]



SIZES = [
    "big",
    "small",
    "large",
    "tiny",
    "huge",
    "massive",
    "miniature",
    "gigantic",
    "vast",
    "slim",
    "narrow",
    "broad",
    "tall",
    "short",
    "long",
    "compact",
    "wide",
    "thin",
    "fat",
    "petite",
    "immense",
    "colossal",
    "little",
    "giant",
    "microscopic",
    "enormous",
    "medium",
    "grand",
    "bulky",
    "mighty",
    "shapely",
    "grandiose",
    "sizable",
    "monstrous",
    "ample",
    "minute",
    "oversized",
    "plump",
    # "under-sized",
    "modest",
    "towering",
    "lofty",
    "stocky",
    "stout",
    "puny",
    "spacious",
    "thick",
    "measly",
]



OPINIONS = [
    "beautiful",
    "ugly",
    "lovely",
    "interesting",
    "boring",
    "fantastic",
    "terrible",
    "wonderful",
    "awful",
    "excellent",
    "bad",
    "good",
    "amazing",
    "dull",
    "charming",
    "disgusting",
    "cute",
    "pleasant",
    "unpleasant",
    "fabulous",
    "mediocre",
    "delightful",
    "horrible",
    "marvelous",
    "magnificent",
    "pathetic",
    "great",
    "poor",
    "impressive",
    "elegant",
    "clumsy",
    "admirable",
    "ridiculous",
    "brilliant",
    "humble",
    "stunning",
    "funny",
    "laughable",
    "absurd",
    "despicable",
    "graceful",
    "remarkable",
    "noteworthy",
    "pitiful",
    "enchanting",
    "grotesque",
    "sublime",
    "lousy",
]
